function q=spcontini(dir,fname,inewpar,varargin)
% SPCONTINI: initialization for "spectral cont": continuation of zero eigenvalue for fold or branch point
%
%  q=spcontini(dir,fname,inewpar)
%  q=spcontini(dir,fname,inewpar,newdir)    - set problem name to newdir
%  q=spcontini(dir,fname,inewpar,newdir,ds) - also set stepsize to ds
%
% * dir/fname = location of file with problem p-struct
% * inewpar=additional parameter required for additional equation solves
% * Note: p.sol.ptype=1 or 2 is required
%
% See also spcontexit
p=loadp(dir,fname);    %generates also p.mat.drop and p.mat.fill
if (p.sol.ptype~=2 && p.sol.ptype~=1)
    fprintf('\nWarning: not a fold or branch point. Set p.sw.spcont to\n'); 
    fprintf('1 (branch point) or 2 (fold point) by hand; the Newton loop\n'); 
    fprintf('will hopefully converge to the desired point\n'); 
    q=p;
end
if(p.sol.ptype==1) 
    fprintf('Initializing branch point continuation.\n'); 
elseif(p.sol.ptype==2) 
    fprintf('Initializing fold point continuation.\n'); 
end
if (p.sw.jac~=1)
    fprintf('\nWarning: Computation may be slow as all derivatives are computed numerically.\n Consider setting jsw=1.\n');
end
if (p.sw.qjac~=1)
    fprintf('\nWarning: Computation may be slow as some pder derivatives are computed numerically.\n Consider defining function qfder.\n');
end
% Compute leading eigenvector
M=getM(p); % M=filltrafo(p,M); 
M=[[M zeros(p.nu,p.nc.nq)]; [zeros(p.nc.nq,p.nu) eye(p.nc.nq)]]; 
Gu=getGu(p,p.u,resi(p,p.u));

%double the size of p.mat.drop and p.mat.fill  
%(these have been generated in loadp for the standard system and need to be doubled for the extended system)
r=size(p.mat.drop,1); s=size(p.mat.drop,2);
if r>1; % HU
  p.mat.drop = [[p.mat.drop zeros(r,s)]; [zeros(r,s) p.mat.drop]]; 
  p.mat.fill = [[p.mat.fill zeros(s,r)]; [zeros(s,r) p.mat.fill]];
end

evopts.disp=0; [phi1,mu1]=eigs(Gu,M,1,0,evopts);

fprintf('Approx. zero eigenvalue=%g.\n',mu1);
phi1=phi1/norm(phi1); % normalize eig-fct. 

% Create new initial vector: format is u=[updeold; updelin; uauxold; auxlin]
% Here auxlin has the length of active aux. vars. except primary par.
p.naux=length(p.u)-p.nu; % store number of auxiliary variables, is used for fold-cont.
p.u=[p.u(1:p.nu);phi1(1:p.nu);p.u(p.nu+1:p.nu+p.naux);phi1(p.nu+1:end)];

% set new active variables including those for linear part (length=p.nc.nq+2+p.nc.nq)
p.nc.ilam=[inewpar; reshape(p.nc.ilam,length(p.nc.ilam),1); [p.naux+1:p.naux+p.nc.nq]'];

% set new equation numbers
p.nc.neq=2*p.nc.neq; p.nu=2*p.nu; p.nc.nq=2*p.nc.nq+1; %HU 
p.sw.spcont=p.sol.ptype; p.sol.restart=1;

fprintf('New active parameters and their values:\n'); printaux(p,1)
% other data initialization
p=resetc(p);
if(~isempty(varargin)); [p,ok]=setfn(p,varargin{1}); if ok~=1; q=p; return; end
else fprintf('warning: file name prefixes unchanged.\n'); end
p.sol.deta=0; p.sol.ineg=-1; p.sw.spcalc=0;
fname=[p.file.pname,sprintf('%i',p.file.count),'.mat'];save(fname,'p'); 
p.file.count=2;q=p;